; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

documentSection("INTERRUPTS")
document("One day, I may add support for hooking your own ISR into the interrupt handler.")

;------ ApdOn ------------------------------------------------------------------
ApdOn:
documentRoutine("ApdOn")
document("Enables the APD function.")
document("Inputs:")
document(" - None")
document("Outputs:")
document(" - None")
document("Destroys:")
document(" - AF")
	ld	a, (mflags + appFlags)
	or	apdEnabledMask
	ld	(mflags + appFlags), a
	ret


;------ ApdOff -----------------------------------------------------------------
ApdOff:
documentRoutine("ApdOff")
document("Disables the APD function.")
document("Inputs:")
document(" - None")
document("Outputs:")
document(" - None")
document("Destroys:")
document(" - AF")
	ld	a, (mflags + appFlags)
	and	apdEnabledMask ^ 255
	ld	(mflags + appFlags), a
	ret


;------ SetUpInterrupts --------------------------------------------------------
SetUpInterrupts:
documentRoutine("SetUpInterrupts")
document("Sets up the IM 2 interrupt system, but does not enable interrupts.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Interrupts set up")
document("Destroys:")
document(" - Assume all")
	di
	xor	a
	out	(pIntMask), a
	out	(pCrstlTmr1Freq), a
	out	(pCrstlTmr2Freq), a
	out	(pCrstlTmr3Freq), a
	out	(pLnkAstSeEnable), a
	; And kill USB
	out	(57h), a
	out	(5Bh), a
	out	(4Ch), a
	ld	a, 2
	out	(54h), a
	; Custom Interrupts
	ld	hl, IvtLocation*256
	ld	de, IvtLocation*256+1
	ld	bc, 256
	ld	a, IvtLocation
	ld	i, a
	ld	a, IsrLocation
	ld	(hl), a
	ldir
#ifdef IS_APPLICATION
	ld	hl, InterruptServiceRoutine
	ld	de, IsrLocation*256+IsrLocation
	;ld	c, (hl)
	;inc	hl
	;ld	b, (hl)
	;inc	hl
	ld	bc, 3
#else
	.error	"Fix ISR loader."
#endif
	ldir
	; Enable only wanted interrupts
	;ld	a, tmrFreq0 | memMapMode0 | battVoltage0
	xor	a	; Above expression evaulates to 0
	out	(pMMapTmrFreq), a
	ld	a, intTmr2 | intOnKey | intDisableLowPowerMode
	out	(pIntMask), a
	im	2
	ret
InterruptServiceRoutine:
;	.dw	3
	jp	realIsr
;InterruptServiceRoutineEnd:


;------ Interrupt Service Routine ----------------------------------------------
	.dw	realIsrEnd-realIsrSource
realIsrSource:
	push	af
	in	a, (pIntId)
	rra
	jr	c, intHOnKey
	rra
	jr	c, +_;RebootMicrOS;intApd
	rra
	jr	c, intHPapd
	rra
	rra
	jr	c, intHLink
;	rra
;	jr	c, intHCrystal
;	rra
;	jr	c, intHHCrystal
	; Wait, there should be no other active interrupt sources.
;	xor	a
;	out	(pIntAck), a
	;call	RebootMicrOS
_:	syscall(RebootMicrOS & 3FFFh)
;intHLink:
;intHCrystal:
;intHApd: ; WTF, didn't we disable this interrupt?
;	call	RebootMicrOS
intHOnKey: ; ON key is for panicing!
	ld	a, intOnKey^255
	out	(pIntAck), a
	ld	a, 0FFh
	out	(pKey), a
	push	bc
	ld	bc, suspendDelay
	ld	(suspendTimer), bc
	pop	bc
	ld	a, 0BFh
	out	(pKey), a
	pop	af
	push	af
	pop	af
	push	af
	in	a, (pKey)
	cp	0BFh
	;jp	z, QuitToEOS
	syscallz(QuitToEOS & 3FFFh)
	cp	7Fh
	;jp	z, RebootMicrOS
	syscallz(RebootMicrOS & 3FFFh)
intHLink:
	ld	a, intLinkport ^ 255
	out	(pIntAck), a
	pop	af
	ei
	ret
intHPapd:
	push	bc
	; ACK the interrupt
	ld	a, intTmr2 ^ 255
	;xor	a
	out	(pIntAck), a
	; General fast timer.  This is useful for timing routines.
	ld	bc, (genFastTimer)
	inc	bc
_:	ld	(genFastTimer), bc
	ld	a, b
	cp	4
	jr	c, +_
	ld	bc, 0
	jr	-_	; Saves one byte.
_:	; Check the cursor timer.  This timer is always running.
	ld	bc, (cursorTimer)
	dec	bc
	ld	(cursorTimer), bc
	ld	a, b
	or	c
	jr	nz, noCursorBlink
	ld	bc, (cursorTimerPeriod)
	ld	(cursorTimer), bc
	ld	a, (mflags + curFlags)
	or	curFlashMask
	ld	(mflags + curFlags), a
	; Check if we're sleepy
	ld	a, (mflags + appFlags)
	bit	apdEnabled, a
	jr	z, noCursorBlink
	ld	bc, (suspendTimer)
	dec	bc
	ld	(suspendTimer), bc
	ld	a, b
	or	c
	jr	nz, noCursorBlink
	ld	bc, suspendDelay
	ld	(suspendTimer), bc
	syscallsave(MicrosApd & 3FFFh)
	pop	bc
	pop	af
	ei
	ret
noCursorBlink:
	; Check if it's time to scan the keyboard
	ld	a, (kbdScanTimer)
	dec	a
	ld	(kbdScanTimer), a
	jr	nz, noScan
	ld	a, kbdScanDivisor
	ld	(kbdScanTimer), a
#ifdef IS_APPLICATION
;	syscallsave(sysScanKeyboard)
	push	de
	push	hl
	ex	af, af'
	push	af
	ld	de, (keyboardPage)
	ld	hl, ScanKeyboard
	call	PagedCall
	pop	af
	ex	af, af'
	pop	hl
	pop	de
#else
	call	ScanKeyboard
#endif
noScan:	pop	bc
	pop	af
	ei
	ret
realIsrEnd: