; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

documentSection("LOW-LEVEL LCD")

;testScreenOn:
;	call	LcdInit
;	call	ResetScreen
;	ld	de, (microsPage0)
;	ld	hl, mHardwareMon
;	jp	Menu

;testScreenOff:
;	call	LcdOff
;	ld	de, (microsPage0)
;	ld	hl, mHardwareMon
;	jp	Menu

;testBacklightReset:
;	call	BacklightReset
;	ld	de, (microsPage0)
;	ld	hl, mHardwareMon
;	jp	Menu

;testBacklightPulse:
;	call	BacklightPulse
;	ld	de, (microsPage0)
;	ld	hl, mHardwareMon
;	jp	Menu

BacklightAdjuster:
	call	HomeUp
	ld	hl, strBacklighAdjuster
	call	PutS
	ld	hl, (currentRow)
	push	hl
	ld	hl, strGetScriptSource-1
	call	PutSClrWind
backlightAdjustLoop:
	pop	hl
	push	hl
	call	Locate
	ld	a, (brightness)
	call	DispByte
	call	GetKey
	cp	skUp
	jr	nz, +_
	call	BacklightUp
	jr	backlightAdjustLoop
_:	cp	skDown
	jr	nz, +_
	call	BacklightDown
	jr	backlightAdjustLoop
_:	cp	skDel
	jr	nz, +_
	call	BacklightReset
	jr	backlightAdjustLoop
_:	cp	skClear
	jr	nz, backlightAdjustLoop
	pop	hl
	ld	de, (microsPage0)
	ld	hl, mHardwareMon
	jp	Menu


;------ ExecScrnScriptUi -------------------------------------------------------
ExecScrnScriptUi:
	call	ClearWind
	ld	hl, strGetScriptSource
	call	PutS
	call	GetHexByte
	ld	ixh, a
	call	GetHexByte
	ld	ixl, a
	call	NewLine
	ld	hl, strGetScriptLogDest
	call	PutS
	call	GetHexByte
	ld	iyh, a
	call	GetHexByte
	ld	iyl, a
	call	ExecScrnCmds
	jp	MainMenu


;------ BacklightOff -----------------------------------------------------------
BacklightOff:
documentRoutine("BacklightOff")
document("Turns the backlight off.  The screen driver itself remains active!")
	in	a, (pGpioData)
	and	0DFh
	out	(pGpioData), a
	ret

	
;------ BacklightUp ------------------------------------------------------------
BacklightUp:
documentRoutine("BacklightUp")
document("Increases the backlight brightness, unless it's already at maximum, in which")
document("case it wraps to the minimum brightness.")
document("This routine will take several milliseconds, because it resets the screen")
document("brightness instead of just assuming it's already correct.")
	ld	a, (brightness)
	or	a
	jr	nz, +_
	ld	a, minBacklightLevel
_:	dec	a
	ld	(brightness), a


;------ BacklightSetLevel ------------------------------------------------------
BacklightSetLevel:
documentRoutine("BacklightSetLevel")
document("Synchronizes the backlight level to match the value in the brightness")
document("variable.")
document("Input:")
document(" - None")
document("Output:")
document(" - Documented effect")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - IX")
	ld	ix, lcdBacklightReset
	call	ExecScrnCmds
	ld	a, (brightness)
	or	a
	ret	z
	ld	e, a
_:	ld	ix, lcdBacklightPulse
	call	ExecScrnCmds
	dec	e
	jr	nz, -_
	ret


;------ BacklightReset ---------------------------------------------------------
BacklightReset:
BacklightOn:
documentRoutine("BacklightOn")
document("Resets the backlight to its maximum brightness.")
document("Input:")
document(" - None")
document("Output:")
document(" - Documented effect")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - IX")
	ld	ix, lcdBacklightReset
	call	ExecScrnCmds
	xor	a
	ld	(brightness), a
	ret


;------ BacklightPulse ---------------------------------------------------------
BacklightPulse:
BacklightDown:
documentRoutine("BacklightDown")
document("Pulses the backlight, thereby dimming it.")
document("Input:")
document(" - None")
document("Output:")
document(" - Documented effect")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - IX")
	ld	ix, lcdBacklightPulse
	call	ExecScrnCmds
	ld	a, (brightness)
	inc	a
	cp	minBacklightLevel
	jr	nc, BacklightReset
	ld	(brightness), a
	ret


;------ LcdOff -----------------------------------------------------------------
LcdOff:
documentRoutine("LcdOff")
document("Turns the screen off.")
document("Input:")
document(" - None")
document("Output:")
document(" - Documented effect")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - IX")
	ld	ix, lcdOffScript
	jp	ExecScrnCmds


;------ LcdOn ------------------------------------------------------------------
;LcdOn:
;	ld	ix, 
;	jp	ExecScrnCmds


;------ LcdInit ----------------------------------------------------------------
LcdInit:
documentRoutine("LcdInit")
document("Resets the screen.")
document("Input:")
document(" - None")
document("Output:")
document(" - Documented effect")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - IX")
	ld	ix, lcdInitScript
	call	ExecScrnCmds
	jp	BacklightSetLevel


;------ ExecScrnCmds -----------------------------------------------------------
escPortLog:
	ld	c, (ix)
	inc	ix
	ld	(iy), 0
	inc	iy
	ld	(iy), c
	inc	iy
	in	a, (c)
	ld	(iy), a
	inc	iy
	jr	ExecScrnCmds
escPortWrite:
	ld	c, (ix)
	inc	ix
	ld	b, (ix)
	inc	ix
	out	(c), b
	jr	ExecScrnCmds
escPortWriteAnd:
	ld	c, (ix)
	inc	ix
	ld	b, (ix)
	inc	ix
	in	a, (c)
	and	b
	out	(c), a
	jr	ExecScrnCmds
escPortWriteOr:
	ld	c, (ix)
	inc	ix
	ld	b, (ix)
	inc	ix
	in	a, (c)
	or	b
	out	(c), a
	jr	ExecScrnCmds
escWriteBlock:
	pushAndDisableInterrupts
	ld	a, (ix)
	inc	ix
	ld	l, (ix)
	inc	ix
	ld	h, (ix)
	inc	ix
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	b, (hl)
	inc	hl
_:	ld	a, (hl)
	inc	hl
	out	(pLcdData), a
	inc	hl
	out	(pLcdData), a
	djnz	-_
	popAndRestoreInterrupts
	jr	ExecScrnCmds
escGpioDirection:
	ld	b, (ix)
	inc	ix
	in	a, (pGpioDirection)
	or	b
	out	(pGpioDirection), a
	jr	ExecScrnCmds
escWait:
	ld	b, (ix)
	inc	ix
escWaitLoop:
	ld	a, (genFastTimer)
	ld	c, a
	ei
	halt
_:	ld	a, (genFastTimer)
	cp	c
	jr	z, -_
	djnz	escWaitLoop
ExecScrnCmds:
documentRoutine("ExecScrnCmds")
document("Processes a block of commands to be sent to the screen.")
document("Useful for testing stuff.")
document("Each command is a command byte, followed by arguments.")
document("Script commands:")
document("scEnd			.equ	0")
document(" - Signals the end of the script.")
document("scWrite			.equ	1")
document(" - Arguments: ")(byte) register, (word) value
document(" - Writes the given 16-bit value to the given port.")
document("scLog			.equ	2")
document(" - Argument: ")(byte) register
document(" - Reads a byte from register and logs it to IY.")
document("scWriteBlock		.equ	3")
document(" - Arguments: ")(byte) register, (word) ptr
document(" - Writes data at ptr to register.  The first byte at ptr is the number of")
document("   words to write.")
document("scWait			.equ	4")
document(" - Argument: ")(byte) time
document(" - Waits for about time/2 milliseconds")
document("scGpioDirection		.equ	5")
document(" - Argument: ")(byte) mask
document(" - The given mask is ORed with port 39.")
document("scGpioAnd		.equ	6")
document(" - Argument: ")(byte) mask
document(" - The value is port 3A is read, masked with mask, and rewritten.")
document("scGpioOr		.equ	7")
document(" - Argument: ")(byte) mask
document(" - The value is port 3A is read, masked with mask, and rewritten.")
document("scPortLog		.equ	8")
document(" - Argument: ")(byte) port
document(" - A value from port is written to the log at IY.")
document("scPortWrite		.equ	9")
document(" - Argument: ")(byte) port, (byte) value
document(" - value is written to port.")
document("scPortWriteAnd		.equ	10")
document(" - Argument: ")(byte) port, (byte) value
document(" - port is read, masked with value, and written back to port.")
document("scPortWriteOr		.equ	11")
document(" - Argument: ")(byte) port, (byte) value
document(" - port is read, masked with value, and written back to port.")
document("Input:")
document(" - IX: Pointer to command list")
document(" - IY: Pointer to location to log data")
document("Output:")
document(" - Command executed")
document("Destroys:")
document(" - AF")
document(" - BC")
document(" - IX")
document(" - HL if block write used")
document(" - IY if logging used")
	ld	a, (ix)
	inc	ix
	or	a
	ret	z
	cp	scWrite
	jp	z, escWrite
	cp	scLog
	jp	z, escLog
	cp	scWriteBlock
	jp	z, escWriteBlock
	cp	scWait
	jp	z, escWait
	cp	scGpioDirection
	jp	z, escGpioDirection
	cp	scGpioAnd
	jp	z, escGpioAnd
	cp	scGpioOr
	jp	z, escGpioOr
	cp	scPortLog
	jp	z, escPortLog
	cp	scPortWrite
	jp	z, escPortWrite
	cp	scPortWriteAnd
	jp	z, escPortWriteAnd
	cp	scPortWriteOr
	jp	z, escPortWriteOr
	jp	RebootMicros
escWrite:
	pushAndDisableInterrupts
	ld	a, (ix)
	inc	ix
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	ld	a, (ix+1)
	inc	ix
	out	(pLcdData), a
	ld	a, (ix-1)
	inc	ix
	out	(pLcdData), a
	popAndRestoreInterrupts
	jr	ExecScrnCmds
escLog:
	pushAndDisableInterrupts
	ld	a, (ix)
	inc	ix
	ld	(iy), a
	inc	iy
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	in	a, (pLcdData)
	ld	(iy+1), a
	inc	iy
	in	a, (pLcdData)
	ld	(iy-1), a
	inc	iy
	inc	ix
	inc	ix
	popAndRestoreInterrupts
	jp	ExecScrnCmds
escGpioAnd:
	ld	b, (ix)
	inc	ix
	in	a, (pGpioData)
	and	b
	out	(pGpioData), a
	jp	ExecScrnCmds
escGpioOr:
	ld	b, (ix)
	inc	ix
	in	a, (pGpioData)
	or	b
	out	(pGpioData), a
	jp	ExecScrnCmds






;------ ClearSrcnFull ----------------------------------------------------------
documentRoutine("ClearSrcnFull")
document("Fully erases the whole screen to black.")
document("Just uses FillScrnFull to do its dirty work.")
ClrScrnFull:
	ld	d, 0
;------ FillScrnFull -----------------------------------------------------------
FillScrnFull:
documentRoutine("FillScrnFull")
document("Fills the entire screen with a specified 16-bit color.")
document("Selected colors:")
document("Component 5/6/5 is the color as the LCD controller sees it.  For consistency,")
document("the middle green channel is divided by two, so each channel is out of 31.")
document("Component 8/8/8 is each channel multiplied by 8 so it is out of 255, like on")
document("PCs.")
document("Hex   Component 5/6/5	Component 8/8/8	Color RGB	Color BGR")
document("00	 0,    0,  0	  0,   0,   0	Black		Black")
document("06	 0,   24,  6	  0, 192,  48	Green		Green")
document("08	 1,    0,  8	  8,   0,  64	Navy Blue	Dark Maroon?")
document("18	 3,    0, 24	 24,   0, 192	Blue		Red")
document("1F	 3,   28, 31	 24, 224, 248	Baby Blue?	Gold/Yellow?")
document("4A	 9,    9, 10")
document("60	12,  1.5,  0	 96,  12,   0	Maroon?		Dark Blue?")
document("33	 0,   12,  3	  0,  96,  24	Dark Green	Better Dark Green")
document("6B	13, 13.5, 11	104, 108,  88	Gray 39%	Gray 39%")
;;; 77	28, 31.5,  7	224, 252,  56	Lime Green	Teal
document("77	 0,   28,  7	  0, 224,  26			Green-screen Green")
document("B5	22, 22.5, 21	176, 180, 168	Gray 68%	Gray 68%")
document("D8	27,    3, 24	216,  24, 192	Magenta?	Blue-Magenta? ")
document("DB	27,   27, 30	216, 216, 240	Gray 88%	Gray 88%")
document("E0	28,  3.5,  0	224,  28,   0	Red		Blue")
document("FF	31,   31, 31	255, 255, 255	White		White")
document("")
document("Input:")
document(" - D: Color to write, high and low bytes are the same")
document("Output:")
document(" - Screen filled with repeating byte")
document(" - Current LCD register set to lrGram")
document(" - Row & Col set to ")(0,0)
document(" - Window set to ")(0,0)-(319,239)
document("Destroys:")
document(" - A, BC")
document("Assumes:")
document(" - Only assumes that the LCD is more or less configured correctly.")
document("Execution Time:")
document(" - Base time is 129 ms")
document(" - Add 31 ms if ASIC-forced LCD delay is set to the minimum of 3 clock cycle")
document(" - Total minimum time is therefore 160 ms (or 6.2 frames/second)")
	pushAndDisableInterrupts ; sigma = 24 c
	ld	c, pLcdCmd	; 7 cc
	xor	a		
	;			; sigma = 11
	; Sync
	out	(pLcdCmd), a	; 11 cc
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	out	(pLcdCmd), a
	;			; sigma = 44
	; lrWinTop
	ld	b, lrWinTop	; 7 cc
	out	(pLcdCmd), a
	out	(c), b		; 12 cc
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 52
	; lrWinBottom
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	ld	a, colorScrnHeight-1
	out	(pLcdData), a
	xor	a
	;			; sigma = 60
	; lrWinLeft
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 49
	; lrWinRight
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	inc	a
	;ld	de, colorScrnWidth ; 10 cc
	out	(pLcdData), a
	ld	a, lcdLow((colorScrnWidth-1))
	out	(pLcdData), a
	xor	a
	;			; sigma = 64
	; lrRow
	ld	b, lrRow
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 52
	; lrCol
	inc	b
	out	(pLcdCmd), a
	out	(c), b
	out	(pLcdData), a
	out	(pLcdData), a
	;			; sigma = 49
	; Now fill the screen
	ld	b, lrGram
	out	(pLcdCmd), a
	out	(c), b
	ld	c, 75
	ld	b, 0
	ld	a, d
	;			; sigma = 51
	; Total time for header:  sigma = 432
	; Plus 3 cycles added by ASIC per write: 30*3 = 90
	; So 522 cc total for initalization.
_:	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	out	(pLcdData), a
	djnz	-_		; 13/8 cc
	; 11*8*256+13*255+8 = 25851 cc
	dec	c
	jr	nz, -_		; 12/7 cc
	; (25851+4)*75+12*74+7 = 1940020
	popAndRestoreInterrupts
	ret			; 10 cc
	; 1940462 = ca. 129 ms
	; Plus 3 cycles per write: 3*2*320*240 = 460800.
	; So, including delays, 2401352 clock cycles + 77 more for save/restore interrupts
	; Which is about 160 ms or 6.2 frames/second.