; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

documentSection("MEMORY")
document("These routines deal with raw memory pointers, writing, and reading.")

;------ CompactPageNumber ------------------------------------------------------
CompactPageNumber:
documentRoutine("CompactPageNumber")
document("Compacts the page number in DE into a single 8-bit number.")
document("WARNING: Passing 80-8F in E will return 80-8F.  Passing 80-8F to")
document("ExpandPageNumber will return 0100-010F.")
document("Input:")
document(" - DE: Page number (D=port 0E, E=port 06)")
document("Output:")
document(" - A: Page number")
document("Destroys:")
document(" - Flags")
;	ld	a, e
;	bit	7, e
;	ret	nz
	ld	a, d
	and	1
	rrca
	or	e
	ret
	

;------ ExpandPageNumber -------------------------------------------------------
ExpandPageNumber:
documentRoutine("ExpandPageNumber")
document("Performs almost the reverse transform of CompacytPageNumber.")
document("The difference is that this routine has no way of knowing whether you wanted")
document("80-8F to be RAM pages, so it always expands them to 0100-010F.")
document("Input:")
document(" - A: Page number")
document("Output:")
document(" - DE: Page number (D=port 0E, E=port 06)")
document("Destroys:")
document(" - AF")
	ld	e, a
	res	7, e
	and	80h
	rlca
	ld	d, a
	ret


;------ LdDEHLInd --------------------------------------------------------------
	.dw	ldDEHLIndEnd-LdDEHLIndSource
LdDEHLIndSource:
documentRoutine("LdDEHLInd")
document("Loads DEHL from the location pointed to by DEHL")
document("Kept in RAM to ensure that it is always quickly available.")
document("Input:")
document(" - DEHL: Pointer to pointer")
document("Output:")
document(" - DEHL: New pointer")
document("Destroys:")
document(" - Nothing")
	push	af
	call	GetByteIncPtr
	push	af
	call	GetByteIncPtr
	ex	(sp), hl
	ld	l, h
	ld	h, a
	ex	(sp), hl
	call	GetByteIncPtr
	push	af
	call	GetByte
	pop	de
	ld	e, d
	ld	d, a
	pop	hl
	pop	af
	ret
ldDEHLIndEnd:


;------ ResolveDE --------------------------------------------------------------
ResolveDE:
sysResolveDE	.equ	internalSyscall(ResolveDE, syscallPage0)
; Resolves DE if HLDE needs to point to data in MicrOS.
; Input:
;  - D: Bit 5 set is DEHL is supposed to point to data in MicrOS.
; Output:
;  - DE: If bit 5 of D was reset, the data are unchanged.
;        If bit 5 of D was set, then the high two bits give a MicrOS page number
;        and DE has that page number.
; Destroys:
;  - AF
	bit	5, d
	ret	z
	ld	a, d
	rlca
	rlca
	and	3
	ld	e, a
	ld	a, (microsBasePage)
	sub	e
	ld	e, a
	ld	a, (microsBasePage+1)
	ld	d, a
	ret	p
	res	7, e
	dec	d
	ret


;------ SetDePageN -------------------------------------------------------------
;SetDePage0:
;sysSetDePage0	.equ	internalSyscall(SetDePage0, syscallPage0)
; Sets DE to have the given MicrOS page number.
; Inputs:
;  - None
; Output:
;  - DE has the page number of a MicrOS page.
; Destroys:
;  - Flags
	ld	de, (microsBasePage)
	ret
;SetDePage3:
;sysSetDePage3	.equ	internalSyscall(SetDePage3, syscallPage0)
;	ld	de, (microsBasePage)
;	call	decDe
	jr	+_
;SetDePage2:
;sysSetDePage2	.equ	internalSyscall(SetDePage2, syscallPage0)
	ld	de, (microsBasePage)
;_:	call	decDe
;	jr	decDe
;SetDePage1:
;sysSetDePage1	.equ	internalSyscall(SetDePage1, syscallPage0)
	ld	de, (microsBasePage)
;decDe:
	dec	e
	ret	p
	ld	e, 7Fh
	dec	d
	ret


;------ IncPtr -----------------------------------------------------------------
	.dw	incPtrEnd-IncPtrSource
IncPtrSource:
documentRoutine("IncPtr")
document("Increments DEHL, wrapping as needed.  If HL points outside of the swappable")
document("memory bank, this does not attempt any wrapping.")
document("Kept in RAM to ensure speed.")
document("Input:")
document(" - DEHL")
document("Output:")
document(" - DEHL+1")
document("Destroys:")
document(" - AF")
	inc	l
	ret	nz
	inc	h
	ld	a, h
	cp	swapTop >> 8
	ret	nz
	sub	40h
	ld	h, a
	inc	e
	ld	a, e
	cp	80h
	ret	nz
	ld	e, 0
	inc	d
	ret
incPtrEnd:


;------ DecPtr -----------------------------------------------------------------
	.dw	decPtrEnd-DecPtrSource
DecPtrSource:
documentRoutine("DecPtr")
document("Decrements DEHL, wrapping as needed.  If HL points outside of the swappable")
document("memory bank, this does not attempt any wrapping.")
document("Kept in RAM to ensure speed.")
document("Input:")
document(" - DEHL")
document("Output:")
document(" - DEHL+1")
document("Destroys:")
document(" - AF")
	ld	a, l
	dec	l
	or	a
	ret	nz	; If L != 0 before, L does not underflow and H is fine.
	ld	a, h
	dec	h
	cp	swapLoc >> 8
	ret	nz
	ld	a, e
	dec	e
	or	a
	jr	z, +_
	cp	80h
	ret	nz
	ld	e, 87h
	ret
_:	ld	e, 7Fh
	ld	a, d
	dec	a
	and	3
	ld	d, a
	ret
decPtrEnd:


;------ AddPtr -----------------------------------------------------------------
	.dw	addPtrEnd-AddPtrSource
AddPtrSource:
documentRoutine("AddPtr")
document("Adds BC to DEHL, wrapping as needed.  THIS WILL SCREW UP if HL points outside")
document("of the swap bank.")
document("Kept in RAM to ensure speed.")
document("Inputs:")
document(" - DEHL: Pointer")
document(" - BC: Amount to add to pointer")
document("Output:")
document(" - DEHL+=BC")
document("Destroys:")
document(" - AF")
#if swapLoc != 4000h
	.error	"Oops, fix the AddPtr function.  It assumes swapping happens at 4000h."
#endif
	ld	a, l
	add	a, c
	ld	l, a
	ret	nc
	ld	a, h
	add	a, b
	ld	h, a
	call	c, addPtrIncDe-AddPtrSource+AddPtr
	ld	a, h
	cp	40h
	call	c, addPtrIncDe-AddPtrSource+AddPtr
	ld	a, h
	cp	0C0h
	call	nc, addPtrIncDe-AddPtrSource+AddPtr
	ld	a, h
	cp	80h
	ret	c
addPtrIncDe:
	ld	a, h
	sub	40h
	ld	h, a
	inc	e
	ld	a, e
	cp	88h
	jr	nc, +_
	cp	80h
	ret	nz
	ld	e, 0
	inc	d
	ret
_:	ld	e, 80h
	ret
addPtrEnd:


;------ SubPtr -----------------------------------------------------------------
	.dw	subPtrEnd-SubPtrSource
SubPtrSource:
documentRoutine("SubPtr")
document("Subtracts BC from DEHL, wrapping as needed.  THIS WILL SCREW UP if HL points")
document("outside of the swap bank.")
document("Kept in RAM to ensure speed.")
document("Inputs:")
document(" - DEHL: Pointer")
document(" - BC: Amount to subtract from pointer")
document("Output:")
document(" - DEHL-=BC")
document("Destroys:")
document(" - AF")
#if swapLoc != 4000h
	.error	"Oops, fix the SubPtr function.  It assumes swapping happens at 4000h."
#endif
	ld	a, l
	sub	c
	ld	l, a
	ret	nc
	ld	a, h
	sub	b
	ld	h, a
	call	c, subPtrDecDe-SubPtrSource+SubPtr
	ld	a, h
	cp	80h
	call	c, subPtrDecDe-SubPtrSource+SubPtr
	ld	a, h
	cp	0C0h
	call	c, subPtrDecDe-SubPtrSource+SubPtr
	ld	a, h
	cp	040h
	ret	nc
subPtrDecDe:
	ld	a, h
	add	a, 40h
	ld	h, a
	dec	e
	ld	a, e
	cp	7Fh
	jr	z, +_
	cp	0FFh
	ret	nz
	ld	e, 7Fh
	dec	d
	ret
_:	ld	e, 87h
	ret
subPtrEnd:


;------ GetByte ----------------------------------------------------------------
	.dw	getByteEnd-GetByteSource
GetByteSource:
documentRoutine("GetByte")
document("Gets a byte from the given location.")
document("Kept in RAM to ensure speed and because it would otherwise require an ugly")
document("solution.")
document("Input:")
document(" - DEHL: Address")
document("Output:")
document(" - A: Byte returned")
document("Destroys:")
document(" - Nothing")
	push	bc
	pushPageA()
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPortHigh), a
	ld	b, (hl)
	popPageA()
	ld	a, b
	pop	bc
	ret
getByteEnd:



;------ GetByteIncPtr ----------------------------------------------------------
	.dw	getByteIncPtrEnd-GetByteIncPtrSource
GetByteIncPtrSource:
documentRoutine("GetByteIncPtr")
document("Calls GetByte and IncPtr.")
	call	GetByte
	push	af
	call	IncPtr
	pop	af
	ret
getByteIncPtrEnd:


;------ PutByte ----------------------------------------------------------------
	.dw	putByteEnd-PutByteSource
PutByteSource:
documentRoutine("PutByte")
document("Writes a byte to the given address, which should be RAM.  If it's not RAM, the")
document("write will probably be ignored.")
document("Kept in RAM because it would otherwise require an ungly solution.")
document("Input:")
document(" - DEHL: Address")
document(" - A: Byte to write")
document("Output:")
document(" - Byte written")
document("Destroys:")
document(" - AF")
	push	bc
	ld	b, a
	pushPageA()
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPortHigh), a
	ld	(hl), b
	popPageA()
	pop	bc
	ret
putByteEnd:


;====== Flash Commands =========================================================
;------ Block Copy/Write -------------------------------------------------------
#ifdef IS_APPLICATION
BlockCopy:
	ld	iy, BlockCopySource-2
	jp	ExecRamCode
	.dw	blockCopyEnd-BlockCopySource
#endif
BlockCopySource:
documentRoutine("BlockCopy")
document("Copies up to 64 K of data from one location to another.  If B is set to")
document("BlockCopyFlashWriteMode, then the writes will be treated as flash writes.")
document("This is a copied-to-RAM-before-execution routine.  You can skip redundant")
document("copying after the first call to this by just calling microsRamCode directly.")
document("Inputs:")
document(" - B': Mode")
document(" - DEHL: Source address")
document(" - DEHL': Destination address")
document(" - IX: Bytes to copy.  If this is 0, it will copy 65536 bytes, not 0.")
document("Output:")
document(" - Data copied")
document(" - DEHL, DEHL' point to byte AFTER last byte")
document("Destroys:")
document(" - AF")
document(" - B, BC'")
document(" - DE, DE'")
document(" - HL, HL'")
document(" - IX")
document("I couldn't think of a use for AF', C, IY, I, or R.")
BlockCopyFlashWriteMode	.equ	1
BlockCopyNoFlashWrite	.equ	0
blockCopyWriteFlash	.equ	0
; A: Read val
; B': Mode
; A': Temp for flash write seq
; DEHL: Read/Write address
	pushAndDisableInterrupts()
	pushPageA()
	res	1, b
	exx
	set	1, b
	exx
blockCopyLoop:
	; Read source byte
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPort|8), a
	ld	a, (hl)
	; Map destination into memory
	exx
	ld	c, a
	ld	a, e
	out	(swapPort), a
	ld	a, d
	out	(swapPort|8), a
	bit	blockCopyWriteFlash, b
	jr	z, noWrite
	; Write-to-flash
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 0A0h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	(hl), c	; Fourth bus cycle---program data
_:	ld	a, (hl)
	xor	c
	bit	7, a
	jr	z, blockCopyIncPtrs
	xor	c	; Don't read twice -> Prevent false failure notification
	bit	5, a	; if bit 5 just happens to be set in the good data.
	jr	z, -_
	; Error!
	ld	a, 0F0h
	ld	(0000h), a
noWrite:
	ld	(hl), c
blockCopyIncPtrs:
	; Inc pointers
	exx
blockCopyIncPtrs2:
	inc	l
	jr	nz, +_
	inc	h
	ld	a, h
	cp	swapTop >> 8
	jr	nz, +_
	;sub	40h
	;ld	h, a
	ld	h, swapLoc >> 8
	inc	e
	ld	a, e
	cp	80h
	jr	nz, +_
	ld	e, 0
	inc	d
_:	exx
	bit	1, b
	jr	nz, blockCopyIncPtrs2
checkIfDone:
	; Are we done?
	dec	ix
	ld	a, ixh
	or	ixl
	jr	nz, blockCopyLoop
	popPageA()
	popAndRestoreInterrupts()
	ret
blockCopyEnd:


;------ Flash write ------------------------------------------------------------
#ifdef IS_APPLICATION
WriteFlashByte:
	ld	iy, WriteFlashByteSource-2
	jp	ExecRamCode
	.dw	writeFlashByteEnd-WriteFlashByteSource
#endif
WriteFlashByteSource:
documentRoutine("WriteFlashByte")
document("Writes a byte to flash.  Flash must be unlocked.  Be aware that pressing CLEAR")
document("will abort the write.")
document("This is a copied-to-RAM-before-execution routine.  You can skip redundant")
document("copying after the first call to this by just calling microsRamCode directly.")
document("Inputs:")
document(" - B: Byte to write")
document(" - D: Page High Bits")
document(" - E: Page")
document(" - HL: Address to write to.  This will not be wrapped.")
document("Outputs:")
document(" - NZ on failure")
document("Destroys:")
document(" - AF, BC")
	; Map address into Z80 memory
	pushAndDisableInterrupts()
	pushPageA()
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	; Flash command sequence
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 0A0h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	(hl), b	; Fourth bus cycle---program data
	; Wait for data to be good
	;ld	a, 0FDh		; This checks for the CLEAR key.
	;out	(pKey), a		; If pressed, it aborts.
	;inc	ix
	;dec	ix
_:	;in	a, (pKey)
	;cp	0BFh
	;jr	z, +_
	ld	a, (hl)
	xor	b
	bit	7, a
	jr	z, programDone
	xor	b
	bit	5, a	; Don't read twice -> Prevent false failure notification if bit 5 just happens to be set in the good data.
	jr	z, -_
_:	ld	a, 0F0h
	ld	(0000h), a
programDone:
	; Fix mappings
	ld	b, a
	popPageA()
	popAndRestoreInterrupts()
	bit	7, b
	ret
writeFlashByteEnd:


;------ Erase sector -----------------------------------------------------------
#ifdef IS_APPLICATION
EraseSector:
	ld	iy, EraseSectorSource-2
	jp	ExecRamCode
	.dw	eraseSectorEnd-EraseSectorSource
#endif
EraseSectorSource:
documentRoutine("EraseSector")
document("Erases a sector of flash.")
document("This is a copied-to-RAM-before-execution routine.  You can skip redundant")
document("copying after the first call to this by just calling microsRamCode directly.")
document("(Although, I question why you would want to erase lots of sectors serially.")
document("Are you trying to destroy user data or brick the calculator?)")
document("Inputs:")
document(" - D: Page High Bits")
document(" - E: Page")
document(" - HL: Address (usually 4000h)")
document("Outputs:")
document(" - Z on fail (this is opposite of writeFlashByte)")
document("Destroys:")
document(" - AF, BC, HL")
	; Map address into Z80 memory
	pushAndDisableInterrupts()
	pushPageA()
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	; Flash command sequence
	ld	a, 0AAh	; First bus cycle---unlock
	ld	(0AAAh), a
	ld	a, 55h	; Second bus cycle---unlock
	ld	(0555h), a
	ld	a, 080h	; Third bus cycle---write command
	ld	(0AAAh), a
	ld	a, 0AAh	; Fourth bus cycle---unlock (again)
	ld	(0AAAh), a
	ld	a, 55h	; Fifth bus cycle---unlock (again)
	ld	(0555h), a
	ld	(hl), 30h
	; Wait for data to be good
	ld	a, 0FDh
	;out	(pKey), a
	;inc	ix
	;dec	ix        
_:	;in	a, (pKey)
	;cp	0BFh
	;jr	z, +_
;	cp	0EFh	; Allow the user to continue using MicrOS during the erase
;	jr	z, eraseDone
	ld	a, (hl)
	bit	7, a
	jr	nz, eraseDone
	bit	5, a
	jr	z, -_
_:	ld	a, 0F0h
	ld	(0000h), a
eraseDone:
	ld	b, a
	; Fix mappings
	popPageA()
	popAndRestoreInterrupts()
	bit	0, b	; bit 0 might be garbage during the erase, but it's definitely 1 after if it succeeded
	ret
eraseSectorEnd:


;------ Autoselect Command Test ------------------------------------------------
#ifdef IS_APPLICATION
IssueAutoselect:
	ld	iy, IssueAutoselect-2
	jp	ExecRamCode
	.dw	issueAutoselectEnd-IssueAutoselectSource
#endif
IssueAutoselectSource:
documentRoutine("IssueAutoselect")
document("This is a copied-to-RAM-before-execution routine.  You can skip redundant")
document("copying after the first call to this by just calling microsRamCode directly.")
document("Inputs:")
document(" - HL: Address")
document(" - D: High Page Bits")
document(" - E: Page")
document("Output:")
document(" - B: Value read")
	pushAndDisableInterrupts()
	pushPageA()
	ld	a, e
	out	(pMPgA), a
	ld	a, d
	out	(pMPgAHigh), a
	; First bus cycle---unlock
	ld	a, 0AAh
	ld	(0AAAh), a
	; Second bus cycle---unlock
	ld	a, 55h
	ld	(0555h), a
	; Third bus cycle---write command
	ld	a, 090h
	ld	(0AAAh), a
	; Read autoselect code
	ld	b, (hl) ;4001h 4047
	ld	a, 0F0h
	ld	(0000h), a
;	ld	l, a
	popPageA()
	popAndRestoreInterrupts()
	ret
issueAutoselectEnd: