; MICROS SYSTEM ROUTINES & EQUATES

;------ RAM Addresses ----------------------------------------------------------
microsRamCode	.equ	$8798
microsBasePage	.equ	$8000


;------ Macros -----------------------------------------------------------------
#macro syscall(routine)
	ld	iy, routine
	call	SysCallRoutine
#endmacro
#macro syscallcc(cc, routine)
	ld	iy, routine
	call	cc, SysCallRoutine
#endmacro
; Total overhead to save and restore interrupts:
; 11 bytes and 77 clock cycles (unless you use the restore alternative macros)
#macro pushAndDisableInterrupts
	ld	a, i
	push	af
	di
#endmacro
#macro popAndRestoreInterrupts
	pop	af
	jp	po, $+4
	ei
#endmacro
#macro popAndRestoreInterruptsRelocatable
	ex	(sp), hl
	bit	2, l
	pop	hl
	jr	z, $+3
	ei
#endmacro
#macro popAndRestoreInterruptsSaveA
	ex	(sp), hl
	bit	2, l
	pop	hl
	jr	z, $+3
	ei
#endmacro
#macro pushPageA
	in	a, (14)
	rra
	in	a, (6)
	push	af
#endmacro
#macro popPageA
	pop	af
	out	(6), a
	sbc	a, a
	out	(14), a
#endmacro


;------ RAM-resident Routines --------------------------------------------------
SysCallRoutine	.equ	$8818
SysJumpRoutine	.equ	$881B
PagedCall	.equ	$881E
PagedJump	.equ	$8821
LdDEHLInd	.equ	$8824
MultHByE	.equ	$8827
PutByte	.equ	$882D
GetByte	.equ	$8830
SubPtr	.equ	$8833
AddPtr	.equ	$8836
DecPtr	.equ	$8839
IncPtr	.equ	$883C
GetByteIncPtr .equ $883F


;------ Page 0 -----------------------------------------------------------------


; Init/exit
StartMicrOS	.equ	$40
RebootMicrOS	.equ	$43
GetVersion	.equ	$46
MainMenu	.equ	$49
RebootDevice	.equ	$4C
MicrosApd	.equ	$4F


; Utility
DispByte	.equ	$52
DispHL	.equ	$55
DispDecimal	.equ	$58
ClearMem	.equ	$5B
ProcessKeyJumpTable	.equ	$5E
Menu	.equ	$61
AnyKey	.equ	$64
Panic	.equ	$67


; System
InitRamRoutines	.equ	$6A


; Interrupts
ApdOn	.equ	$6D
ApdOff	.equ	$70
SetUpInterrupts	.equ	$73


; Memory
CompactPageNumber	.equ	$76
ExpandPageNumber	.equ	$79
; FYI, these routines are copied to RAM before execution.
; After the first call, the last such routine executed remains in microsRamCode,
; so you can call the routine again faster by just directly calling microsRamCode.
BlockCopy	.equ	$7C
WriteFlashByte	.equ	$7F
EraseSector	.equ	$82
IssueAutoSelect	.equ	$85


; Keyboard
GetAscii	.equ	$88
ClearShiftFlags	.equ	$8B
ClearAllShiftFlags	.equ	$8E
GetKeyShifts	.equ	$91
GetHexByte	.equ	$94
GetKey	.equ	$97
CursorOn	.equ	$9A
CursorOff	.equ	$9D
GetCSC	.equ	$A0
PeekCSC	.equ	$A3
ScanKeyboard	.equ	$A6
RawGetCSC	.equ	$A9


; LCD
ExecScrnScriptUi	.equ	$AC
BacklightOff	.equ	$AF
BacklightUp	.equ	$B2
BacklightSetLevel	.equ	$B5
BacklightOn	.equ	$B8
BacklightDown	.equ	$BB
LcdOff	.equ	$BE
LcdInit	.equ	$C1
ClrScrnFull	.equ	$C4
FillScrnFull	.equ	$C7


; Screen
ScrollUp	.equ	$CA
SetFullScrnWind	.equ	$CD
ClearWind	.equ	$D0
PutSClrWind	.equ	$D3
PutS	.equ	$D6
Locate	.equ	$D9
HomeUp	.equ	$DC
CursorBack	.equ	$DF
CursorForward	.equ	$E2
CursorRight	.equ	$E5
CursorLeft	.equ	$E8
CursorDown	.equ	$EB
CursorUp	.equ	$EE
AdvanceCursor	.equ	$F1
NewLine	.equ	$F4
NewLineClrEOL	.equ	$F7
NewLine2	.equ	$FA
NewLineClrEOL2	.equ	$FD
NextTab	.equ	$100
PutSpace	.equ	$103
PutSpaces	.equ	$106
PutC2	.equ	$109
PutC	.equ	$10C
ReadChar	.equ	$10F
ResetScreen	.equ	$112
FixCursor	.equ	$115
SetCharRow	.equ	$118
SetCharCol	.equ	$11B
SetLcdWindowColumnBounds	.equ	$11E
SetDirectionRight	.equ	$121
SetDirectionDown	.equ	$124
