; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.

documentSection("UTILITIES")
document("Contains various helpful routines.")

;====== Math Routines ==========================================================
;------ Milos Bazelides routines -----------------------------------------------
; These routines are by Milos "baze" Bazelides, baze_at_baze_au_com
; Retrived from http://baze.au.com/misc/z80bits.html 25 October 2008
	.dw	multHByEEnd-MultHByESource
MultHByESource:
documentRoutine("MultHByE")
document("Standard optimized 8-bit multiply.  Kept in RAM for best speed.")
document("Input: H = Multiplier, E = Multiplicand")
document("Output: HL = Product")
	xor	a
	ld	l, a
	ld	d, a
	sla	h		; optimised 1st iteration
	jr	nc,$+3
	ld	l,e
	;
	ld	b, 7
multHbyEloop:
	add	hl,hl		; unroll 7 times
	jr	nc,$+3		; ...
	add	hl,de		; ...
	djnz	multHbyEloop
	ret
multHByEEnd:


;------ ------------------------------------------------------------------------
DispHL:
documentRoutine("DispHL")
document("Input:")
document(" - HL")
document("Output:")
document(" - Word displayed")
document("Destroys:")
document(" - AF")
	ld	a, h
	call	DispByte
	ld	a, l
DispByte:
documentRoutine("DispByte")
document("Input:")
document(" - A: Byte")
document("Output:")
document(" - Byte displayed")
document("Destroys:")
document(" - AF")
	push	af
	rra
	rra
	rra
	rra
	call	+_
	pop	af
_:	or	0F0h
	daa
	add	a, 0A0h
	adc	a, 40h
	call	PutC
	ret


;------ ------------------------------------------------------------------------
DispDecimal:
documentRoutine("DispDecimal")
document("Input:")
document(" - HL")
document("Output:")
document(" - 16-bit number displayed in decimal")
document("Destroys:")
document(" - BC")
document(" - AF")
	ld	bc,-10000
	call	+_
	ld	bc,-1000
	call	+_
	ld	bc,-100
	call	+_
	ld	c,-10
	call	+_
	ld	c,b
_:	ld	a,'0'-1
_:	inc	a
	add	hl,bc
	jr	c, -_
	sbc	hl,bc
	call	PutC
	ret


;====== Routines ===============================================================


;------ ------------------------------------------------------------------------
ClearMem:
documentRoutine("ClearMem")
document("Clears a section of memory SLOW! :p")
document("Input:")
document(" - HL: Location to kill")
document(" - B: Number of bytes to kill")
document(" - A: What to clear it with")
	ld	(hl), a
	inc	hl
	djnz	ClearMem
	ret


;------ Process Key Jump Table -------------------------------------------------
ProcessKeyJumpTable:
documentRoutine("ProcessKeyJumpTable")
document("Processes a jump table in which each five byte entry contains a key value")
document("followed by a 32-bit pointer to jump to if that key matches A.  Otherwise,")
document("this routine returns to the sender.  (If the jump is taken, four stack entries")
document("are popped off the stack, 1 from caller and 3 from syscall.)")
document("Inputs:")
document(" - DEHL: Table")
document(" - A: Keycode")
	ld	c, a
_:	call	GetByte;ld	a, (hl)
	call	IncPtr;inc	hl
	or	a
	ret	z
	cp	c
	jr	z, +_
	call	IncPtr
	call	IncPtr
	call	IncPtr
	call	IncPtr
	jr	-_
_:	;ld	a, (hl)
	;inc	hl
	;ld	h, (hl)
	;ld	l, a
	;pop	af
	;xor	a
	;jp	(hl)
	pop	af
	pop	af
	pop	af
	pop	af
	call	LdDEHLInd
	call	ResolveDE
	xor	a
	jp	PagedJump


;------ Menu -------------------------------------------------------------------
Menu:
documentRoutine("Menu")
document("Simple menu routine.")
document("DEHL points to a data structure that describes the menu.")
document("The first item in the menu is a zero-termed string to be displayed.")
document("After that, there is a list of acceptable keys and 32-bit jump addresses.")
document("This is a JUMP not a CALL!")
document("This passes the key that was pressed in B.")
	ld	a, (mflags + appFlags)
	and	textAutoScrollMask ^ 255
	ld	(mflags + appFlags), a
	push	hl
	push	de
;	call	ClearWind
	call	HomeUp
	pop	de
	pop	hl
	call	PutSClrWind ; hl = byte after 0
	push	hl
	push	de
	exx
	pop	de
	pop	hl
	exx
menuKeyLoop:
	call	GetKey	; Should preserve all in current MicrOS
	exx
	push	hl
	push	de
	exx
	pop	de
	pop	hl
	ld	b, a
	call	GetByte ; sanity check
	or	a ;
	jp	z, MainMenu
_:	; Key table scan loop
	call	GetByteIncPtr
	or	a
	jr	z, menuKeyLoop
	cp	b
	jr	z, +_
	call	IncPtr
	call	IncPtr
	call	IncPtr
	call	IncPtr
	jr	-_
_:	; Found an entry!  Is it a link to another menu?
	call	LdDEHLInd
	call	ResolveDE
	call	GetByte
	or	a
	jr	z, +_
	;jp	(hl)
	jp	PagedJump
_:	; Yes, it's linking to another menu.
	call	IncPtr
	jp	Menu


;------ AnyKey -----------------------------------------------------------------
AnyKey:
documentRoutine("AnyKey")
document("Waits until a key is pressed, then returns.  This routine, unlike GetKey, does")
document("not depend on the interrupt-based keyboard driver.")
document("Inputs:")
document(" - None")
document("Output:")
document(" - Waits until a key is pressed, then returns")
document("Destroys:")
document(" - AF")
document(" - BC")
	ld	bc, 0
	ld	a, 0FFh
	out	(pKey), a
	djnz	$
	inc	a
	out	(pKey), a
	djnz	$
_:	in	a, (pKey)
	inc	a
	jr	z, -_
	ld	a, 0FFh
	out	(pKey), a
_:	djnz	$
	dec	c
	jr	nz, -_
	ret